@@ -1,5 +1,5 @@
-/* -*- Mode: C; tab-width: 4; indent-tabs-mode: t; c-basic-offset: 4 -*- */
-/***************************************************************************
+/*
+ * network-manager-openvpn - OpenVPN integration with NetworkManager
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -15,9 +15,9 @@
  * with this program; if not, write to the Free Software Foundation, Inc.,
  * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  *
- * Copyright (C) 2008 - 2013 Dan Williams <dcbw@redhat.com> and Red Hat, Inc.
- *
- **************************************************************************/
+ * Copyright (C) 2008 - 2013 Dan Williams <dcbw@redhat.com>
+ * Copyright (C) 2008 - 2018 Red Hat, Inc.
+ */
 
 #include "nm-default.h"
 
@@ -41,6 +41,7 @@
 #define INLINE_BLOB_CA                  NMV_OVPN_TAG_CA
 #define INLINE_BLOB_CERT                NMV_OVPN_TAG_CERT
 #define INLINE_BLOB_EXTRA_CERTS         NMV_OVPN_TAG_EXTRA_CERTS
+#define INLINE_BLOB_CRL_VERIFY          NMV_OVPN_TAG_CRL_VERIFY
 #define INLINE_BLOB_KEY                 NMV_OVPN_TAG_KEY
 #define INLINE_BLOB_PKCS12              NMV_OVPN_TAG_PKCS12
 #define INLINE_BLOB_SECRET              NMV_OVPN_TAG_SECRET
@@ -775,8 +776,7 @@ inline_blob_write_out (const InlineBlobData *data, GError **error)
 NMConnection *
 do_import (const char *path, const char *contents, gsize contents_len, GError **error)
 {
-	gs_unref_object NMConnection *connection_free = NULL;
-	NMConnection *connection;
+	gs_unref_object NMConnection *connection = NULL;
 	NMSettingConnection *s_con;
 	NMSettingIPConfig *s_ip4;
 	NMSettingVpn *s_vpn;
@@ -792,12 +792,13 @@ do_import (const char *path, const char *contents, gsize contents_len, GError **
 	const char *ta_direction = NULL, *secret_direction = NULL;
 	gboolean allow_ta_direction = FALSE, allow_secret_direction = FALSE;
 	gboolean have_certs, have_ca;
-	GSList *inline_blobs = NULL, *sl_iter;
+	GSList *inline_blobs = NULL;
+	GSList *sl_iter;
 
 	g_return_val_if_fail (!error || !*error, NULL);
 
 	connection = nm_simple_connection_new ();
-	connection_free = connection;
+
 	s_con = NM_SETTING_CONNECTION (nm_setting_connection_new ());
 	nm_connection_add_setting (connection, NM_SETTING (s_con));
 	s_ip4 = NM_SETTING_IP_CONFIG (nm_setting_ip4_config_new ());
@@ -915,7 +916,7 @@ do_import (const char *path, const char *contents, gsize contents_len, GError **
 			if (!args_params_check_nargs_minmax (params, 0, 1, &line_error))
 				goto handle_line_error;
 			if (params[1]) {
-				if (!args_params_parse_int64 (params, 1, 1, G_MAXINT32, &v_int64, &line_error))
+				if (!args_params_parse_int64 (params, 1, 0, G_MAXINT32, &v_int64, &line_error))
 					goto handle_line_error;
 				setting_vpn_add_data_item_int64 (s_vpn, NM_OPENVPN_KEY_MSSFIX, v_int64);
 			} else
@@ -1007,7 +1008,7 @@ do_import (const char *path, const char *contents, gsize contents_len, GError **
 
 			if (nm_streq (v, "no")) {
 				/* old plasma-nm used to set "comp-lzo=no" to mean unset, thus it spoiled
-				 * to "no" option to be used in the connection. Workaround that, by instead
+				 * the "no" option to be used in the connection. Workaround that, by instead
 				 * using "no-by-default" (bgo#769177). */
 				v = "no-by-default";
 			} else if (!NM_IN_STRSET (v, "yes", "adaptive")) {
@@ -1018,6 +1019,21 @@ do_import (const char *path, const char *contents, gsize contents_len, GError **
 			continue;
 		}
 
+		if (NM_IN_STRSET (params[0], NMV_OVPN_TAG_COMPRESS)) {
+			if (!args_params_check_nargs_minmax (params, 0, 1, &line_error))
+				goto handle_line_error;
+			if (params[1]) {
+				if (!NM_IN_STRSET (params[1], "lzo", "lz4", "lz4-v2")) {
+					line_error = g_strdup_printf (_("unsupported compress argument"));
+					goto handle_line_error;
+				}
+				setting_vpn_add_data_item (s_vpn, NM_OPENVPN_KEY_COMPRESS, params[1]);
+			} else {
+				setting_vpn_add_data_item (s_vpn, NM_OPENVPN_KEY_COMPRESS, "yes");
+			}
+			continue;
+		}
+
 		if (NM_IN_STRSET (params[0], NMV_OVPN_TAG_FLOAT)) {
 			if (!args_params_check_nargs_n (params, 0, &line_error))
 				goto handle_line_error;
@@ -1205,6 +1221,24 @@ do_import (const char *path, const char *contents, gsize contents_len, GError **
 			continue;
 		}
 
+		if (NM_IN_STRSET (params[0], NMV_OVPN_TAG_TLS_VERSION_MIN)){
+			if (!args_params_check_nargs_n (params, 1, &line_error))
+				goto handle_line_error;
+			if (!args_params_check_arg_utf8 (params, 1, NULL, &line_error))
+				goto handle_line_error;
+			setting_vpn_add_data_item (s_vpn, NM_OPENVPN_KEY_TLS_VERSION_MIN, params[1]);
+			continue;
+		}
+
+		if (NM_IN_STRSET (params[0], NMV_OVPN_TAG_TLS_VERSION_MAX)){
+			if (!args_params_check_nargs_n (params, 1, &line_error))
+				goto handle_line_error;
+			if (!args_params_check_arg_utf8 (params, 1, NULL, &line_error))
+				goto handle_line_error;
+			setting_vpn_add_data_item (s_vpn, NM_OPENVPN_KEY_TLS_VERSION_MAX, params[1]);
+			continue;
+		}
+
 		if (NM_IN_STRSET (params[0],
 		                  NMV_OVPN_TAG_CA,
 		                  NMV_OVPN_TAG_CERT,
@@ -1464,6 +1498,8 @@ do_import (const char *path, const char *contents, gsize contents_len, GError **
 				key = NM_OPENVPN_KEY_KEY;
 			else if (nm_streq (token, INLINE_BLOB_EXTRA_CERTS))
 				key = NM_OPENVPN_KEY_EXTRA_CERTS;
+			else if (nm_streq (token, INLINE_BLOB_CRL_VERIFY))
+				key = NM_OPENVPN_KEY_CRL_VERIFY_FILE;
 			else if (nm_streq (token, INLINE_BLOB_PKCS12)) {
 				is_base64 = TRUE;
 				key = NULL;
@@ -1639,28 +1675,50 @@ handle_line_error:
 		}
 	}
 
-	inline_blobs = g_slist_reverse (inline_blobs);
-	for (sl_iter = inline_blobs; sl_iter; sl_iter = sl_iter->next) {
-		const InlineBlobData *data = sl_iter->data;
+	if (inline_blobs) {
+		GSList *tmp_list = NULL;
 
-		/* Check whether the setting was not overwritten by a later entry in the config-file. */
-		if (nm_streq (data->token, INLINE_BLOB_PKCS12)) {
-			if (   !setting_vpn_eq_data_item_utf8safe (s_vpn, NM_OPENVPN_KEY_CA, data->path)
-			    && !setting_vpn_eq_data_item_utf8safe (s_vpn, NM_OPENVPN_KEY_CERT, data->path)
-			    && !setting_vpn_eq_data_item_utf8safe (s_vpn, NM_OPENVPN_KEY_KEY, data->path))
-				continue;
-		} else {
-			if (!setting_vpn_eq_data_item_utf8safe (s_vpn, data->key, data->path))
+		/* filter out blobs that are shadowed and not used. */
+		while ((sl_iter = inline_blobs)) {
+			InlineBlobData *data = sl_iter->data;
+			gboolean is_good = TRUE;
+
+			inline_blobs = inline_blobs->next;
+
+			/* Check whether the setting was not overwritten by a later entry in the config-file. */
+			if (nm_streq (data->token, INLINE_BLOB_PKCS12)) {
+				if (   !setting_vpn_eq_data_item_utf8safe (s_vpn, NM_OPENVPN_KEY_CA, data->path)
+				    && !setting_vpn_eq_data_item_utf8safe (s_vpn, NM_OPENVPN_KEY_CERT, data->path)
+				    && !setting_vpn_eq_data_item_utf8safe (s_vpn, NM_OPENVPN_KEY_KEY, data->path))
+					is_good = FALSE;
+			} else {
+				if (!setting_vpn_eq_data_item_utf8safe (s_vpn, data->key, data->path))
+					is_good = FALSE;
+			}
+			if (!is_good) {
+				g_slist_free_1 (sl_iter);
+				inline_blob_data_free (data);
 				continue;
+			}
+
+			/* we prepend the element to a new, temporary list. That means, we
+			 * reverse the order here. That is just what we need, because the
+			 * list was previously constructed in reverse order. */
+			sl_iter->next = tmp_list;
+			tmp_list = sl_iter;
 		}
+		inline_blobs = tmp_list;
+	}
+
+	for (sl_iter = inline_blobs; sl_iter; sl_iter = sl_iter->next) {
 		if (!inline_blob_write_out (sl_iter->data, error))
 			goto out_error;
 	}
+
 	g_slist_free_full (inline_blobs, (GDestroyNotify) inline_blob_data_free);
 
-	connection_free = NULL;
 	g_return_val_if_fail (!error || !*error, connection);
-	return connection;
+	return g_steal_pointer (&connection);
 
 out_error:
 	g_slist_free_full (inline_blobs, (GDestroyNotify) inline_blob_data_free);
@@ -1956,6 +2014,12 @@ do_export_create (NMConnection *connection, const char *path, GError **error)
 		args_write_line (f, NMV_OVPN_TAG_COMP_LZO, value);
 	}
 
+	value = nm_setting_vpn_get_data_item (s_vpn, NM_OPENVPN_KEY_COMPRESS);
+	if (nm_streq0 (value, "yes"))
+		args_write_line (f, NMV_OVPN_TAG_COMPRESS);
+	else if (value)
+		args_write_line_setting_value (f, NMV_OVPN_TAG_COMPRESS, s_vpn, NM_OPENVPN_KEY_COMPRESS);
+
 	if (nm_streq0 (nm_setting_vpn_get_data_item (s_vpn, NM_OPENVPN_KEY_FLOAT), "yes"))
 		args_write_line (f, NMV_OVPN_TAG_FLOAT);
 
@@ -2058,6 +2122,14 @@ do_export_create (NMConnection *connection, const char *path, GError **error)
 			                 nm_utils_str_utf8safe_unescape (key, &s_free));
 		}
 
+		key = nm_setting_vpn_get_data_item (s_vpn, NM_OPENVPN_KEY_TLS_VERSION_MIN);
+		if (nmovpn_arg_is_set (key))
+			args_write_line (f, NMV_OVPN_TAG_TLS_VERSION_MIN, key);
+
+		key = nm_setting_vpn_get_data_item (s_vpn, NM_OPENVPN_KEY_TLS_VERSION_MAX);
+		if (nmovpn_arg_is_set (key))
+			args_write_line (f, NMV_OVPN_TAG_TLS_VERSION_MAX, key);
+
 		key = nm_setting_vpn_get_data_item (s_vpn, NM_OPENVPN_KEY_EXTRA_CERTS);
 		if (nmovpn_arg_is_set (key)) {
 			gs_free char *s_free = NULL;
