@@ -1,14 +1,5 @@
-/* -*- Mode: C; tab-width: 4; indent-tabs-mode: t; c-basic-offset: 4 -*- */
-/***************************************************************************
- * CVSID: $Id: nm-openvpn.c 4232 2008-10-29 09:13:40Z tambeti $
- *
- * nm-openvpn.c : GNOME UI dialogs for configuring openvpn VPN connections
- *
- * Copyright (C) 2005 Tim Niemueller <tim@niemueller.de>
- * Copyright (C) 2008 Tambet Ingo, <tambet@gmail.com>
- * Copyright (C) 2008 - 2010 Dan Williams, <dcbw@redhat.com>
- * Copyright (C) 2008 - 2017 Red Hat, Inc.
- * Based on work by David Zeuthen, <davidz@redhat.com>
+/*
+ * network-manager-openvpn - OpenVPN integration with NetworkManager
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -24,7 +15,12 @@
  * with this program; if not, write to the Free Software Foundation, Inc.,
  * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  *
- **************************************************************************/
+ * Based on work by David Zeuthen, <davidz@redhat.com>
+ * Copyright (C) 2005 Tim Niemueller <tim@niemueller.de>
+ * Copyright (C) 2008 Tambet Ingo, <tambet@gmail.com>
+ * Copyright (C) 2008 - 2010 Dan Williams, <dcbw@redhat.com>
+ * Copyright (C) 2008 - 2017 Red Hat, Inc.
+ */
 
 #include "nm-default.h"
 
@@ -56,38 +52,6 @@ static GtkFileFilter *sk_file_chooser_filter_new (void);
 
 /*****************************************************************************/
 
-static const char *comp_lzo_values[] = {
-	"adaptive",
-	"yes",
-	"no-by-default",
-};
-
-static const char *
-comp_lzo_values_conf_coerce (const char *value_conf)
-{
-	if (!value_conf || nm_streq (value_conf, "no"))
-		return NULL;
-	if (nm_streq (value_conf, "yes"))
-		return "yes";
-	if (nm_streq (value_conf, "no-by-default"))
-		return "no-by-default";
-	return "adaptive";
-}
-
-static const char *
-comp_lzo_values_conf_to_display (const char *value_conf)
-{
-	if (nm_streq (value_conf, "yes"))
-		return "yes";
-	if (nm_streq (value_conf, "no-by-default"))
-		return "no";
-	if (nm_streq (value_conf, "adaptive"))
-		return "adaptive";
-	g_return_val_if_reached ("adaptive");
-}
-
-/*****************************************************************************/
-
 /* From gnome-control-center/panels/network/connection-editor/ui-helpers.c */
 
 static void
@@ -684,6 +648,7 @@ sk_file_chooser_filter_new (void)
 static const char *const advanced_keys[] = {
 	NM_OPENVPN_KEY_AUTH,
 	NM_OPENVPN_KEY_CIPHER,
+	NM_OPENVPN_KEY_COMPRESS,
 	NM_OPENVPN_KEY_COMP_LZO,
 	NM_OPENVPN_KEY_CONNECT_TIMEOUT,
 	NM_OPENVPN_KEY_CRL_VERIFY_DIR,
@@ -717,6 +682,8 @@ static const char *const advanced_keys[] = {
 	NM_OPENVPN_KEY_TA_DIR,
 	NM_OPENVPN_KEY_TLS_CRYPT,
 	NM_OPENVPN_KEY_TLS_REMOTE,
+	NM_OPENVPN_KEY_TLS_VERSION_MIN,
+	NM_OPENVPN_KEY_TLS_VERSION_MAX,
 	NM_OPENVPN_KEY_TUNNEL_MTU,
 	NM_OPENVPN_KEY_TUN_IPV6,
 	NM_OPENVPN_KEY_VERIFY_X509_NAME,
@@ -1463,10 +1430,10 @@ advanced_dialog_new (GHashTable *hash, const char *contype)
 	GtkListStore *store;
 	GtkTreeIter iter;
 	int vint;
-	guint i;
 	guint32 active;
 	NMSettingSecretFlags pw_flags;
 	GError *error = NULL;
+	NMOvpnComp comp;
 
 	g_return_val_if_fail (hash != NULL, NULL);
 
@@ -1584,24 +1551,14 @@ advanced_dialog_new (GHashTable *hash, const char *contype)
 	_builder_init_optional_spinbutton (builder, "fragment_checkbutton", "fragment_spinbutton", !!value,
 	                                   _nm_utils_ascii_str_to_int64 (value, 10, 0, 65535, 1300));
 
+	comp = nmovpn_compression_from_options (g_hash_table_lookup (hash, NM_OPENVPN_KEY_COMP_LZO),
+	                                        g_hash_table_lookup (hash, NM_OPENVPN_KEY_COMPRESS));
 
-	value = comp_lzo_values_conf_coerce (g_hash_table_lookup (hash, NM_OPENVPN_KEY_COMP_LZO));
-	widget = GTK_WIDGET (_builder_init_toggle_button (builder, "lzo_checkbutton", value != NULL));
-	combo = GTK_WIDGET (gtk_builder_get_object (builder, "lzo_combo"));
-	store = gtk_list_store_new (1, G_TYPE_STRING);
-	active = 0;
-	for (i = 0; i < G_N_ELEMENTS (comp_lzo_values); i++) {
-		gtk_list_store_append (store, &iter);
-		gtk_list_store_set (store, &iter,
-		                    0, comp_lzo_values_conf_to_display (comp_lzo_values[i]),
-		                    -1);
-		if (nm_streq (comp_lzo_values[i], value ?: "adaptive"))
-			active = i;
-	}
-	gtk_combo_box_set_model (GTK_COMBO_BOX (combo), GTK_TREE_MODEL (store));
-	g_object_unref (store);
-	gtk_combo_box_set_active (GTK_COMBO_BOX (combo), active);
+	combo = GTK_WIDGET (gtk_builder_get_object (builder, "compress_combo"));
+	widget = GTK_WIDGET (_builder_init_toggle_button (builder, "compress_checkbutton", comp != NMOVPN_COMP_DISABLED));
 	g_object_bind_property (widget, "active", combo, "sensitive", G_BINDING_SYNC_CREATE);
+	if (comp != NMOVPN_COMP_DISABLED)
+		gtk_combo_box_set_active (GTK_COMBO_BOX (combo), comp - 1);
 
 	_builder_init_toggle_button (builder, "mssfix_checkbutton", _hash_get_boolean (hash, NM_OPENVPN_KEY_MSSFIX));
 	_builder_init_toggle_button (builder, "float_checkbutton", _hash_get_boolean (hash, NM_OPENVPN_KEY_FLOAT));
@@ -1816,6 +1773,17 @@ advanced_dialog_new (GHashTable *hash, const char *contype)
 	_builder_init_optional_spinbutton (builder, "max_routes_checkbutton", "max_routes_spinbutton", !!value,
 	                                   _nm_utils_ascii_str_to_int64 (value, 10, 0, 100000000, 100));
 
+	value = g_hash_table_lookup (hash, NM_OPENVPN_KEY_TLS_VERSION_MIN);
+	if (value && *value) {
+		widget = GTK_WIDGET (gtk_builder_get_object (builder, "tls_version_min"));
+		gtk_entry_set_text (GTK_ENTRY (widget), value);
+	}
+	value = g_hash_table_lookup (hash, NM_OPENVPN_KEY_TLS_VERSION_MAX);
+	if (value && *value) {
+		widget = GTK_WIDGET (gtk_builder_get_object (builder, "tls_version_max"));
+		gtk_entry_set_text (GTK_ENTRY (widget), value);
+	}
+
 	return dialog;
 }
 
@@ -1827,7 +1795,6 @@ advanced_dialog_new_hash_from_dialog (GtkWidget *dialog)
 	GtkBuilder *builder;
 	const char *contype = NULL;
 	const char *value;
-	int active;
 	int proxy_type = PROXY_TYPE_NONE;
 	GtkTreeModel *model;
 	GtkTreeIter iter;
@@ -1934,12 +1901,19 @@ advanced_dialog_new_hash_from_dialog (GtkWidget *dialog)
 		}
 	}
 
-	widget = GTK_WIDGET (gtk_builder_get_object (builder, "lzo_checkbutton"));
+	widget = GTK_WIDGET (gtk_builder_get_object (builder, "compress_checkbutton"));
 	if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (widget))) {
-		combo = GTK_WIDGET (gtk_builder_get_object (builder, "lzo_combo"));
-		active = gtk_combo_box_get_active (GTK_COMBO_BOX (combo));
-		if (active >= 0 && active < G_N_ELEMENTS (comp_lzo_values))
-			g_hash_table_insert (hash, NM_OPENVPN_KEY_COMP_LZO, g_strdup (comp_lzo_values[active]));
+		const char *opt_compress;
+		const char *opt_comp_lzo;
+		NMOvpnComp comp;
+
+		combo = GTK_WIDGET (gtk_builder_get_object (builder, "compress_combo"));
+		comp = gtk_combo_box_get_active (GTK_COMBO_BOX (combo)) + 1;
+		nmovpn_compression_to_options (comp, &opt_comp_lzo, &opt_compress);
+		if (opt_compress)
+			g_hash_table_insert (hash, NM_OPENVPN_KEY_COMPRESS, g_strdup (opt_compress));
+		if (opt_comp_lzo)
+			g_hash_table_insert (hash, NM_OPENVPN_KEY_COMP_LZO, g_strdup (opt_comp_lzo));
 	}
 
 	widget = GTK_WIDGET (gtk_builder_get_object (builder, "mssfix_checkbutton"));
@@ -2017,6 +1991,15 @@ advanced_dialog_new_hash_from_dialog (GtkWidget *dialog)
 		if (hmacauth)
 			g_hash_table_insert (hash, NM_OPENVPN_KEY_AUTH, hmacauth);
 	}
+	entry = GTK_WIDGET (gtk_builder_get_object (builder, "tls_version_min"));
+	value = gtk_entry_get_text (GTK_ENTRY (entry));
+	if (value && *value)
+		g_hash_table_insert (hash, NM_OPENVPN_KEY_TLS_VERSION_MIN, g_strdup (value));
+
+	entry = GTK_WIDGET (gtk_builder_get_object (builder, "tls_version_max"));
+	value = gtk_entry_get_text (GTK_ENTRY (entry));
+	if (value && *value)
+		g_hash_table_insert (hash, NM_OPENVPN_KEY_TLS_VERSION_MAX, g_strdup (value));
 
 	contype = g_object_get_data (G_OBJECT (dialog), "connection-type");
 	if (   !strcmp (contype, NM_OPENVPN_CONTYPE_TLS)
@@ -2696,6 +2679,7 @@ nm_vpn_editor_factory_openvpn (NMVpnEditorPlugin *editor_plugin,
                                NMConnection *connection,
                                GError **error)
 {
+	g_type_ensure (NMA_TYPE_CERT_CHOOSER);
 	return openvpn_editor_new (connection, error);
 }
 #endif
